/*
    Tucnak - VHF contest log
    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>

    This program is free software; you can redistribute it and/or                                                        
    modify it under the terms of the GNU General Public License                                                          
    version 2 as published by the Free Software Foundation.

*/

#include "header.h"

#ifdef HAVE_HAMLIB

struct trig *trig;

struct trig *init_trig(){
	struct trig *trig;
    int i, fds[0];

    //dbg("init_trig()\n");
    if (!cfg->rig_model) return NULL;
    if (!cfg->rig_filename || !*cfg->rig_filename) return NULL;
    rig_set_debug(RIG_DEBUG_BUG);

	trig = g_new0(struct trig, 1);
    trig->pipe_write = -1;
    trig->qrg = 0.0;
        trig->qrg = 144200001.2; //todo
    
    if (pipe(fds)) internal("Can't create pipe");
    trig->pipe_write=fds[1];
    trig->pid=fork();
    if (!trig->pid){
        close(0);
        dup(fds[0]);
        close(1);
        dup(tpipe->threadpipe_write);
        for (i=3; i<1024; i++) close(i);
        trig_main((rig_model_t)cfg->rig_model, cfg->rig_filename);
        exit(0);
    }
    if (trig->pid<0) {
        close(trig->pipe_write);
        log_addf("Can't fork for trig\n");
        g_free(trig);
        return NULL;
    }
    dbg("trig->pid=%d\n", trig->pid);
	return trig;
}

void free_trig(struct trig *trig){
    int status;
    
    if (!trig) return;

    if (!trig->pid) return;
    ST_START;
    if (trig->pid) kill(trig->pid, SIGINT);
    waitpid(trig->pid, &status, 0);
    ST_STOP;
  /*  dbg("done\n");*/
    trig->pid=0;
	if (trig->pipe_write>=0) close(trig->pipe_write);
	g_free(trig);
}

void trig_read_handler(struct trig *trig, char *s){
    struct zstring *zs;
    char *cmd;
    
    zs = zstrdup(s);
    
    cmd = ztokenize(zs, 1);
//     dbg("trig_read_handler   rcvd: '%s'  cmd='%s'\n", s, cmd);
    if (strcmp(cmd, "!")==0){  /* error */
        log_addf("Rig error: %s", ztokenize(zs, 0));
    }
    if (strcasecmp(cmd, "qrg")==0){
        trig->qrg = atof(ztokenize(zs,0)) + cfg->rig_lo;
    }
    zfree(zs);
    redraw_later();
}


void trig_set_mode(struct trig *trig, rmode_t mode){
    char s[40];

    if (!trig) return;
    g_snprintf(s, sizeof(s), "MODE;%d;\n", (int)mode);
    write(trig->pipe_write, s, strlen(s));
}

void trig_set_qrg(struct trig *trig, double qrg){
    char s[40];

    if (!trig) return;
    g_snprintf(s, sizeof(s), "FREQ;%f;\n", qrg - cfg->rig_lo);
    write(trig->pipe_write, s, strlen(s));
}

void trig_resend_freq(struct trig *trig){
    char s[40];

    if (!trig) return;
    g_snprintf(s, sizeof(s), "RESENDFREQ;\n");
    write(trig->pipe_write, s, strlen(s));
}

int trig_main(rig_model_t model, char *filename){
    char s[256];
    int ret;
    RIG *rig;
    freq_t freq, oldfreq;
    

    dbg("trig_main\n");
    signal(SIGINT, SIG_DFL);
    signal(SIGSEGV, SIG_DFL);

    freq = oldfreq = (freq_t)0;

    rig = rig_init(model);
    if (!rig){
        printf("TRIG;!;Can't init rig %d\n", (int)model);
        write(tpipe->threadpipe_write, s, strlen(s));
        return 1;
    }
    safe_strncpy(rig->state.rigport.pathname, filename, FILPATHLEN);
    ret = rig_open(rig);
    if (ret != RIG_OK){
        printf("TRIG;!;Can't open rig %d (%s)\n", (int)model, rig->state.rigport.pathname);
        rig_cleanup(rig);
        return 2;
    }
    
    if (fcntl(0, F_SETFL, O_NONBLOCK)){
        printf("TRIG;e;Can't set O_NONBLOCK\n");
        return 3;
    }
    
    while(1){
        int status;
        struct zstring *zs;
        char *cmd;
        
        s[sizeof(s)-1]='\0';
        if (fgets(s, sizeof(s)-1, stdin) != NULL){
            //fprintf(stderr, "s=%s\n", s);
            zs = zstrdup(s);
            cmd = ztokenize(zs, 1);

            if (strcasecmp(cmd, "MODE")==0){
                rmode_t mode;

                mode = (rmode_t)atoi(ztokenize(zs, 0));
                status = rig_set_mode(rig, RIG_VFO_CURR, mode, RIG_PASSBAND_NORMAL);
                fprintf(stderr, "mode=%d status=%d\n", mode, status);
            }

            if (strcasecmp(cmd, "FREQ")==0){
                freq_t fr;
                
                sscanf(ztokenize(zs, 0), "%"SCNfreq, &fr);
                status = rig_set_freq(rig, RIG_VFO_CURR, fr);
                fprintf(stderr, "freq=%"PRIfreq"\n", fr);
            }

            if (strcasecmp(cmd, "RESENDFREQ")==0){
                printf("TRIG;qrg;%"PRIfreq"\n", freq);
            }
            zfree(zs);
        }
        
        status = rig_get_freq(rig, RIG_VFO_CURR, &freq);
        if (status == RIG_OK){  
            //dbg("TRIG;qrg;%"PRIfreq"\n", freq);
            if (freq != oldfreq){
                oldfreq = freq;
                printf("TRIG;qrg;%"PRIfreq"\n", freq);
            }
        }  
    }
    return 0;
}

#endif
