/*
    Tucnak - VHF contest log
    Copyright (C) 2002-2006  Ladislav Vaiz <ok1zia@nagano.cz>

    This program is free software; you can redistribute it and/or                                                        
    modify it under the terms of the GNU General Public License                                                          
    version 2 as published by the Free Software Foundation.

*/

#include "header.h"

int winkey_init(struct cwdaemon *cwda){
    cwda->fd = -1;
    winkey_open(cwda, 1);
	return 0;
}

int winkey_open(struct cwdaemon *cwda, int verbose){
	struct termios tio;
    char errbuf[1024];
    
    if (cwda->fd >= 0) return 0;
    // TODO lock ttyS
    cwda->fd = open(cfg->cwda_device, O_RDWR | O_NOCTTY);
    if (cwda->fd < 0){
        if (verbose) log_addf("Can't open %s %s", cfg->cwda_device, strerror_r(errno, errbuf, sizeof(errbuf)));
        winkey_reset(cwda);
        return -1;
    }
    
	if (fcntl(cwda->fd,F_SETFL,O_NONBLOCK)) {
		if (verbose) log_addf("Can't set O_NONBLOCK on %s: %s",cfg->cwda_device, strerror_r(errno, errbuf, sizeof(errbuf)) );
        return -4;
    }
    
	tcgetattr(cwda->fd,&tio);
	tio.c_cflag=B1200|CS8|CLOCAL|CREAD|CSTOPB;
	tio.c_iflag=INPCK;
	tio.c_lflag=0;
	tio.c_oflag=0;
	tio.c_cc[VMIN]=1;
	tio.c_cc[VTIME]=5;
    
	tcsetattr(cwda->fd,TCSANOW,&tio);
    
	set_handlers(cwda->fd, winkey_read_handler, NULL, NULL, (union cba_t)cwda);
    return 0;
}

int winkey_free(struct cwdaemon *cwda){
    if (!cwda) return 0;

    if (cwda->fd>=0){
		set_handlers(cwda->fd, NULL, NULL, NULL, (union cba_t)cwda);
        close(cwda->fd);
        cwda->fd=-1;
    }
	return 0;
}

int winkey_reset(struct cwdaemon *cwda){

	write(cwda->fd, "\x0a", 1);
	return 0;
}

int winkey_cw(struct cwdaemon *cwda, int onoff){
	return 0;
}

int winkey_ptt(struct cwdaemon *cwda, int onoff){
    char s[16];
    
    if (cwda->fd<0) return 0;
    sprintf(s,"\x18%c", onoff ? '\x01' : '\x00');
    write(cwda->fd, s, 2);
	return 0;
}

int winkey_ssbway(struct cwdaemon *cwda, int onoff){
	return 0;
}

int winkey_text(struct cwdaemon *cwda, char *text){
    if (!cwda || cwda->fd<0) return 0;

    write(cwda->fd, text, strlen(text)); 
    return 0;
}

int winkey_speed(struct cwdaemon *cwda, int wpm){
    char s[16];
    
    if (cwda->fd<0) return 0;

    /*dbg("speed=%d\n", wpm);*/
    sprintf(s,"\x02%c", wpm);
    write(cwda->fd, s, 2);
    return 0;
}

int winkey_weight(struct cwdaemon *cwda, int weight){
    char s[16];
    
    if (cwda->fd<0) return 0;

    sprintf(s,"\x03%c", 32 /* weight*/);
    write(cwda->fd, s, 2);
    return 0;
}

int winkey_tune(struct cwdaemon *cwda, int tune){
	char s[16];

	sprintf(s, "\x0b%c", tune ? '\x01' : '\x00');
    write(cwda->fd, s, 2);
	return 0;
}

void winkey_read_handler(union cba_t cba){
    char s[1024];
    int ret, i;
    struct cwdaemon *cwda;

    dbg("winkey_read_handler\n");
    cwda = cba.cwda;

	ret = read(cwda->fd, s, 1000);
    if (ret<=0) return;
	for (i=0; i<ret; i++){
		dbg("%02x ", (unsigned char)s[i]);
	}
	dbg("\n");
}

